@@ -2,9 +2,9 @@
  *
  * roqet.c - Rasqal RDF Query test program
  *
- * $Id: roqet.c,v 1.48 2004/10/26 23:35:21 cmdjb Exp $
+ * $Id: roqet.c,v 1.70 2005/03/10 00:05:20 cmdjb Exp $
  *
- * Copyright (C) 2004, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2004-2005, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
  * University of Bristol, UK http://www.bristol.ac.uk/
  * 
@@ -90,7 +90,7 @@ rdql_parser_error(const char *msg)
 #endif
 
 
-#define GETOPT_STRING "cdho:i:qs:v"
+#define GETOPT_STRING "cdf:ho:i:nqs:vw"
 
 #ifdef HAVE_GETOPT_LONG
 static struct option long_options[] =
@@ -98,12 +98,15 @@ static struct option long_options[] =
   /* name, has_arg, flag, val */
   {"count", 0, 0, 'c'},
   {"dump-query", 0, 0, 'd'},
+  {"dryrun", 0, 0, 'n'},
+  {"format", 1, 0, 'f'},
   {"help", 0, 0, 'h'},
   {"output", 1, 0, 'o'},
   {"input", 1, 0, 'i'},
   {"quiet", 0, 0, 'q'},
   {"source", 1, 0, 's'},
   {"version", 0, 0, 'v'},
+  {"walk-query", 0, 0, 'w'},
   {NULL, 0, 0, 0}
 };
 #endif
@@ -113,169 +116,33 @@ static int error_count=0;
 
 static const char *title_format_string="Rasqal RDF query utility %s\n";
 
-#define FILE_READ_BUF_SIZE 2048
+#ifdef BUFSIZ
+#define FILE_READ_BUF_SIZE BUFSIZ
+#else
+#define FILE_READ_BUF_SIZE 1024
+#endif
 
 #define MAX_QUERY_ERROR_REPORT_LEN 512
 
 
-static int
-roqet_xml_print_xml_attribute(FILE *handle,
-                              unsigned char *attr, unsigned char *value,
-                              void *user_data, 
-                              raptor_simple_message_handler handler)
-{
-  size_t attr_len;
-  size_t len;
-  size_t escaped_len;
-  unsigned char *buffer;
-  unsigned char *p;
-  
-  attr_len=strlen((const char*)attr);
-  len=strlen((const char*)value);
-
-  escaped_len=raptor_xml_escape_string(value, len,
-                                       NULL, 0, '"',
-                                       handler,
-                                       user_data);
-
-  buffer=(unsigned char*)malloc(1 + attr_len + 2 + escaped_len + 1 +1);
-  if(!buffer)
-    return 1;
-  p=buffer;
-  *p++=' ';
-  strncpy((char*)p, (const char*)attr, attr_len);
-  p+= attr_len;
-  *p++='=';
-  *p++='"';
-  raptor_xml_escape_string(value, len,
-                           p, escaped_len, '"',
-                           handler, user_data);
-  p+= escaped_len;
-  *p++='"';
-  *p++='\0';
-  
-  fputs((const char*)buffer, handle);
-  free(buffer);
-
-  return 0;
-}
-
-
-
 static int
 roqet_query_results_print_as_xml(rasqal_query_results *results, FILE *fh,
-                                 raptor_simple_message_handler handler,
-                                 void *user_data)
+                                 raptor_uri* base_uri)
 {
-  fputs("<results xmlns=\"http://www.w3.org/sw/2001/DataAccess/result1#\">\n\n",
-        fh);
+  raptor_iostream *iostr;
+  raptor_uri* uri;
   
+  iostr=raptor_new_iostream_to_file_handle(fh);
+  if(!iostr)
+    return 1;
+  
+  uri=raptor_new_uri((const unsigned char*)"http://www.w3.org/TR/2004/WD-rdf-sparql-XMLres-20041221/");
+  rasqal_query_results_write(iostr, results, uri, base_uri);
 
-  while(!rasqal_query_results_finished(results)) {
-    int i;
-
-    fputs("  <result>\n", fh);
-    for(i=0; i<rasqal_query_results_get_bindings_count(results); i++) {
-      const unsigned char *name=rasqal_query_results_get_binding_name(results, i);
-      rasqal_literal *l=rasqal_query_results_get_binding_value(results, i);
-      int print_end=1;
-      size_t len;
-      int is_xml=0;
-      
-      if(!l)
-        continue;
-      
-      fputs("    <", fh);
-      fputs((const char*)name, fh);
-
-      switch(l->type) {
-        case RASQAL_LITERAL_URI:
-          if(roqet_xml_print_xml_attribute(fh, (unsigned char*)"uri",
-                                           raptor_uri_as_string(l->value.uri),
-                                           user_data, 
-                                           handler))
-            return 1;
-          
-          fputs("/>\n", fh);
-          print_end=0;
-          break;
-        case RASQAL_LITERAL_STRING:
-          len=strlen((const char*)l->string);
-          if(!len) {
-            fputs("/>\n", fh);
-            print_end=0;
-            break;
-          }
-          
-          if(l->language) {
-            if(roqet_xml_print_xml_attribute(fh, (unsigned char*)"xml:lang",
-                                             (unsigned char *)l->language,
-                                             user_data,
-                                             handler))
-              return 1;
-          }
-
-          if(l->datatype) {
-            if(!strcmp((const char*)raptor_uri_as_string(l->datatype),
-                       (const char*)raptor_xml_literal_datatype_uri_string))
-              is_xml=1;
-            else {
-              if(roqet_xml_print_xml_attribute(fh, (unsigned char*)"datatype",
-                                               raptor_uri_as_string(l->datatype),
-                                               user_data, 
-                                               handler))
-                return 1;
-            }
-          }
-          
-          fputc('>', fh);
-
-          if(is_xml)
-            fputs((const char*)l->string, fh);
-          else {
-            int xml_string_len=raptor_xml_escape_string(l->string, len,
-                                                        NULL, 0, 0,
-                                                        NULL, NULL);
-            if(xml_string_len == (int)len)
-              fputs((const char*)l->string, fh);
-            else {
-              unsigned char *xml_string=(unsigned char*)malloc(xml_string_len+1);
-              
-              xml_string_len=raptor_xml_escape_string(l->string, len,
-                                                      xml_string, xml_string_len, 0,
-                                                      NULL, NULL);
-              fputs((const char*)xml_string, fh);
-              free(xml_string);
-            }
-          }
-          
-          break;
-        case RASQAL_LITERAL_BLANK:
-        case RASQAL_LITERAL_PATTERN:
-        case RASQAL_LITERAL_QNAME:
-        case RASQAL_LITERAL_INTEGER:
-        case RASQAL_LITERAL_BOOLEAN:
-        case RASQAL_LITERAL_FLOATING:
-        case RASQAL_LITERAL_VARIABLE:
-        default:
-          fprintf(stderr, "%s: Cannot turn literal type %d into XML", 
-                  program, l->type);
-          exit(1);
-      }
-
-      if(print_end) {
-        fputs("</", fh);
-        fputs((const char*)name, fh);
-        fputs(">\n", fh);
-      }
-    }
-    fputs("  </result>\n\n", fh);
-    
-    rasqal_query_results_next(results);
-  }
-
-  fputs("</results>\n", fh);
+  raptor_free_uri(uri);
 
+  raptor_free_iostream(iostr);
+  
   return 0;
 }
 
@@ -292,32 +159,102 @@ roqet_error_handler(void *user_data,
 }
 
 
+static const char *spaces="                                                                                  ";
+
 static void
-roqet_simple_message_handler(void *user_data, const char *message, ...)
-{
-  va_list arguments;
+roqet_graph_pattern_walk(rasqal_graph_pattern *gp, int gp_index,
+                         FILE *fh, int indent) {
+  int triple_index=0;
+  int flags;
+      
+  flags=rasqal_graph_pattern_get_flags(gp);
+  
+  fwrite(spaces, sizeof(char), indent, fh);
+  fprintf(fh, "graph pattern %d with flags %d\n", gp_index, flags);
+  
+  indent+= 2;
 
-  va_start(arguments, message);
+  while(1) {
+    rasqal_triple* t=rasqal_graph_pattern_get_triple(gp, triple_index);
+    if(!t)
+      break;
+    
+    fwrite(spaces, sizeof(char), indent, fh);
+    fprintf(fh, "triple %d: ", triple_index);
+    rasqal_triple_print(t, fh);
+    fputc('\n', fh);
 
-  fprintf(stderr, "%s: Error - ", program);
-  vfprintf(stderr, message, arguments);
-  fputc('\n', stderr);
+    triple_index++;
+  }
 
-  error_count++;
+  /* look for sub graph patterns */
+  gp_index=0;
+  while(1) {
+    rasqal_graph_pattern* sgp=rasqal_graph_pattern_get_sub_graph_pattern(gp, gp_index);
+    if(!sgp)
+      break;
+
+    roqet_graph_pattern_walk(sgp, gp_index, fh, indent);
+    gp_index++;
+  }
+
+  if(gp_index > 0) {
+    fwrite(spaces, sizeof(char), indent, fh);
+    fprintf(fh, "found %d sub-graph patterns\n", gp_index);
+  }
+
+  /* look for constraints */
+  gp_index=0;
+  while(1) {
+    rasqal_expression* expr=rasqal_graph_pattern_get_constraint(gp, gp_index);
+    if(!expr)
+      break;
+
+    fwrite(spaces, sizeof(char), indent, fh);
+    fprintf(fh, "constraint %d: ", triple_index);
+    rasqal_expression_print(expr, fh);
+    fputc('\n', fh);
+
+    gp_index++;
+  }
+
+  fputc('\n', fh);
 
-  va_end(arguments);
 }
 
 
+    
+static void
+roqet_query_walk(rasqal_query *rq, FILE *fh) {
+  int gp_index=0;
+  int i;
+
+  i=rasqal_query_get_distinct(rq);
+  if(i != 0)
+    fprintf(fh, "query asks for distinct results\n");
+  i=rasqal_query_get_limit(rq);
+  if(i >= 0)
+    fprintf(fh, "query asks for result limits %d\n", i);
+  
+  while(1) {
+    rasqal_graph_pattern* gp=rasqal_query_get_graph_pattern(rq, gp_index);
+    if(!gp)
+      break;
+    
+    roqet_graph_pattern_walk(gp, gp_index, fh, 2);
+    gp_index++;
+  }
+}
+
+
+
 int
 main(int argc, char *argv[]) 
 { 
-  char *query_string=NULL;
+  void *query_string=NULL;
   unsigned char *uri_string=NULL;
   int free_uri_string=0;
   unsigned char *base_uri_string=NULL;
-  unsigned char *source_uri_string=NULL;
-  int free_source_uri_string=0;
   rasqal_query *rq;
   rasqal_query_results *results;
   char *ql_name="rdql";
@@ -325,7 +262,6 @@ main(int argc, char *argv[])
   int rc=0;
   raptor_uri *uri=NULL;
   raptor_uri *base_uri=NULL;
-  raptor_uri *source_uri=NULL;
   char *filename=NULL;
   char *p;
   int usage=0;
@@ -333,7 +269,12 @@ main(int argc, char *argv[])
   int quiet=0;
   int count=0;
   int dump_query=0;
-  
+  int dryrun=0;
+  int walk_query=0;
+  raptor_sequence* source_uris=NULL;
+  raptor_serializer* serializer=NULL;
+  const char *serializer_syntax_name="ntriples";
+
   program=argv[0];
   if((p=strrchr(program, '/')))
     program=p+1;
@@ -347,6 +288,8 @@ main(int argc, char *argv[])
   while (!usage && !help)
   {
     int c;
+    raptor_uri *source_uri;
+    
 #ifdef HAVE_GETOPT_LONG
     int option_index = 0;
 
@@ -371,10 +314,38 @@ main(int argc, char *argv[])
         dump_query=1;
         break;
 
+      case 'f':
+        if(optarg) {
+          if(raptor_serializer_syntax_name_check(optarg))
+            serializer_syntax_name=optarg;
+          else {
+            int i;
+            
+            fprintf(stderr, "%s: invalid argument `%s' for `" HELP_ARG(o, output) "'\n",
+                    program, optarg);
+            fprintf(stderr, "Valid arguments are:\n");
+            for(i=0; 1; i++) {
+              const char *help_name;
+              const char *help_label;
+              if(raptor_serializers_enumerate(i, &help_name, &help_label, NULL, NULL))
+                break;
+              printf("  %-12s for %s\n", help_name, help_label);
+            }
+            usage=1;
+            break;
+            
+          }
+        }
+        break;
+
       case 'h':
         help=1;
         break;
 
+      case 'n':
+        dryrun=1;
+        break;
+
       case 'o':
         if(optarg) {
           if(!strcmp(optarg, "simple"))
@@ -384,7 +355,7 @@ main(int argc, char *argv[])
           else {
             fprintf(stderr, "%s: invalid argument `%s' for `" HELP_ARG(o, output) "'\n",
                     program, optarg);
-            fprintf(stderr, "Valid arguments are:\n  `simple'   for a simple format (default)\n  `xml'      for an experimental XML format\n");
+            fprintf(stderr, "Valid arguments are:\n  `simple'   for a simple format (default)\n  `xml'      for SPARQL variable bindings XML format\n");
             usage=1;
           }
         }
@@ -415,13 +386,39 @@ main(int argc, char *argv[])
         break;
 
       case 's':
-        source_uri_string=(unsigned char*)optarg;
+        if(!source_uris) {
+          source_uris=raptor_new_sequence((raptor_sequence_free_handler*)raptor_free_uri,
+                                          (raptor_sequence_print_handler*)raptor_sequence_print_uri);
+          if(!source_uris) {
+            fprintf(stderr, "%s: Failed to create source sequence\n", program);
+          return(1);
+          }
+        }
+        
+        if(!access((const char*)optarg, R_OK)) {
+          unsigned char* source_uri_string=raptor_uri_filename_to_uri_string((const char*)optarg);
+          source_uri=raptor_new_uri(source_uri_string);
+          raptor_free_memory(source_uri_string);
+        } else
+          source_uri=raptor_new_uri((const unsigned char*)optarg);
+
+        if(!source_uri) {
+          fprintf(stderr, "%s: Failed to create source URI for %s\n",
+                  program, optarg);
+          return(1);
+        }
+        raptor_sequence_push(source_uris, source_uri);
         break;
 
       case 'v':
         fputs(rasqal_version_string, stdout);
         fputc('\n', stdout);
         exit(0);
+
+      case 'w':
+        walk_query=1;
+        break;
+
     }
     
   }
@@ -451,27 +448,41 @@ main(int argc, char *argv[])
     puts("Run an RDF query giving variable bindings or RDF triples.");
     puts("\nMain options:");
     puts(HELP_TEXT("h", "help            ", "Print this help, then exit"));
+    puts(HELP_TEXT("f FORMAT", "format FORMAT", "Set graph result format to one of:"));
+    for(i=0; 1; i++) {
+      const char *help_name;
+      const char *help_label;
+      if(raptor_serializers_enumerate(i, &help_name, &help_label, NULL, NULL))
+        break;
+      printf("    %-15s         %s", help_name, help_label);
+      if(!i)
+        puts(" (default)");
+      else
+        putchar('\n');
+    }
     puts(HELP_TEXT("i", "input LANGUAGE  ", "Set query language name to one of:"));
     for(i=0; 1; i++) {
       const char *help_name;
       const char *help_label;
       if(rasqal_languages_enumerate(i, &help_name, &help_label, NULL))
         break;
-      printf("    %-12s            %s", help_name, help_label);
+      printf("    %-15s         %s", help_name, help_label);
       if(!i)
         puts(" (default)");
       else
         putchar('\n');
     }
-    puts(HELP_TEXT("o", "output FORMAT   ", "Set output format to one of:"));
-    puts("    'simple'                A simple format (default)");
-    puts("    'xml'                   An experimental XML format");
+    puts(HELP_TEXT("o", "output FORMAT   ", "Set variable binding result format to one of:"));
+    puts("    simple                  A simple text format (default)");
+    puts("    xml                     SPARQL variable bindings XML format");
     puts("\nAdditional options:");
     puts(HELP_TEXT("c", "count           ", "Count triples - no output"));
     puts(HELP_TEXT("d", "dump-query      ", "Dump the parsed query"));
+    puts(HELP_TEXT("n", "dryrun          ", "Prepare but do not run the query"));
     puts(HELP_TEXT("q", "quiet           ", "No extra information messages"));
     puts(HELP_TEXT("s", "source URI      ", "Query against RDF data at source URI"));
     puts(HELP_TEXT("v", "version         ", "Print the Rasqal version"));
+    puts(HELP_TEXT("d", "walk-query      ", "Walk the prepared query using the API"));
     puts("\nReport bugs to <redland-dev@lists.librdf.org>.");
     puts("Rasqal home page: http://librdf.org/rasqal/");
     exit(0);
@@ -521,41 +532,42 @@ main(int argc, char *argv[])
     }
   }
 
-  if(source_uri_string) {
-    if(!access((const char*)source_uri_string, R_OK)) {
-      source_uri_string=raptor_uri_filename_to_uri_string((const char*)source_uri_string);
-      free_source_uri_string=1;
-    }
-    source_uri=raptor_new_uri(source_uri_string);
-    if(!source_uri) {
-      fprintf(stderr, "%s: Failed to create source URI for %s\n",
-              program, source_uri_string);
-      return(1);
-    }
-  }
-
   if(!uri_string) {
-    query_string=(char*)calloc(FILE_READ_BUF_SIZE, 1);
+    query_string=calloc(FILE_READ_BUF_SIZE, 1);
     fread(query_string, FILE_READ_BUF_SIZE, 1, stdin);
   } else if(filename) {
+    raptor_stringbuffer *sb=raptor_new_stringbuffer();
+    size_t len;
     FILE *fh;
-    query_string=(char*)calloc(FILE_READ_BUF_SIZE, 1);
+
     fh=fopen(filename, "r");
-    if(fh) {
-      fread(query_string, FILE_READ_BUF_SIZE, 1, fh);
-      fclose(fh);
-    } else {
+    if(!fh) {
       fprintf(stderr, "%s: file '%s' open failed - %s", 
               program, filename, strerror(errno));
       rc=1;
       goto tidy_setup;
     }
+    
+    while(!feof(fh)) {
+      unsigned char buffer[FILE_READ_BUF_SIZE];
+      size_t read_len;
+      read_len=fread((char*)buffer, 1, FILE_READ_BUF_SIZE, fh);
+      if(read_len > 0)
+        raptor_stringbuffer_append_counted_string(sb, buffer, read_len, 1);
+      if(read_len < FILE_READ_BUF_SIZE)
+        break;
+    }
+    fclose(fh);
+
+    len=raptor_stringbuffer_length(sb);
+    query_string=malloc(len+1);
+    raptor_stringbuffer_copy_to_string(sb, (unsigned char*)query_string, len);
+    raptor_free_stringbuffer(sb);
   } else {
     raptor_www *www=raptor_www_new();
     if(www) {
       raptor_www_set_error_handler(www, roqet_error_handler, NULL);
-      raptor_www_fetch_to_string(www, uri, (void**)&query_string, NULL,
-                                 malloc);
+      raptor_www_fetch_to_string(www, uri, &query_string, NULL, malloc);
       raptor_www_free(www);
     }
     if(!query_string || error_count) {
@@ -584,9 +596,23 @@ main(int argc, char *argv[])
   }
   
   rq=rasqal_new_query((const char*)ql_name, (const unsigned char*)ql_uri);
+  rasqal_query_set_error_handler(rq, NULL, roqet_error_handler);
+  rasqal_query_set_fatal_error_handler(rq, NULL, roqet_error_handler);
+  
+  if(source_uris) {
+    while(raptor_sequence_size(source_uris)) {
+      raptor_uri* source_uri=(raptor_uri*)raptor_sequence_pop(source_uris);
+      if(rasqal_query_add_data_graph(rq, source_uri, source_uri, 
+                                     RASQAL_DATA_GRAPH_NAMED)) {
+        fprintf(stderr, "%s: Failed to add named graph %s\n", program, 
+                raptor_uri_as_string(source_uri));
+      }
+      raptor_free_uri(source_uri);
+    }
+  }
 
   if(rasqal_query_prepare(rq, (const unsigned char*)query_string, base_uri)) {
-    size_t len=strlen(query_string);
+    size_t len=strlen((const char*)query_string);
     
     fprintf(stderr, "%s: Parsing query '", program);
     if(len > MAX_QUERY_ERROR_REPORT_LEN) {
@@ -600,16 +626,19 @@ main(int argc, char *argv[])
     goto tidy_query;
   }
 
-  if(source_uri) {
-    rasqal_query_add_source(rq, source_uri);
-    source_uri=NULL;
+  if(walk_query) {
+    fprintf(stderr, "%s: walking query structure\n", program);
+    roqet_query_walk(rq, stdout);
   }
-
+  
   if(dump_query) {
     fprintf(stderr, "Query:\n");
     rasqal_query_print(rq, stdout);
   }
 
+  if(dryrun)
+    goto tidy_query;
+
   if(!(results=rasqal_query_execute(rq))) {
     fprintf(stderr, "%s: Query execution failed\n", program);
     rc=1;
@@ -617,37 +646,80 @@ main(int argc, char *argv[])
   }
 
   if(output_format == OUTPUT_FORMAT_XML) {
-    fprintf(stderr, "%s: WARNING - This XML format is very likely to change, do not rely on it\n", program);
-    roqet_query_results_print_as_xml(results, stdout, 
-                                     roqet_simple_message_handler, NULL);
+    roqet_query_results_print_as_xml(results, stdout, base_uri);
   } else {
-    while(!rasqal_query_results_finished(results)) {
-      int i;
+    if(rasqal_query_results_is_bindings(results)) {
+      if(!quiet)
+        fprintf(stdout, "%s: Query returned bindings results:\n", program);
 
-      fputs("result: [", stdout);
-      for(i=0; i<rasqal_query_results_get_bindings_count(results); i++) {
-        const unsigned char *name=rasqal_query_results_get_binding_name(results, i);
-        rasqal_literal *value=rasqal_query_results_get_binding_value(results, i);
+      while(!rasqal_query_results_finished(results)) {
+        int i;
 
-        if(i>0)
-          fputs(", ", stdout);
-        fprintf(stdout, "%s=", name);
-        if(value)
-          rasqal_literal_print(value, stdout);
-        else
-          fputs("NULL", stdout);
+        fputs("result: [", stdout);
+        for(i=0; i<rasqal_query_results_get_bindings_count(results); i++) {
+          const unsigned char *name=rasqal_query_results_get_binding_name(results, i);
+          rasqal_literal *value=rasqal_query_results_get_binding_value(results, i);
+
+          if(i>0)
+            fputs(", ", stdout);
+          fprintf(stdout, "%s=", name);
+          if(value)
+            rasqal_literal_print(value, stdout);
+          else
+            fputs("NULL", stdout);
+        }
+        fputs("]\n", stdout);
+
+        rasqal_query_results_next(results);
       }
-      fputs("]\n", stdout);
 
-      rasqal_query_results_next(results);
+      if(!quiet)
+        fprintf(stderr, "%s: Query returned %d results\n", program, 
+                rasqal_query_results_get_count(results));
+
+    } else if (rasqal_query_results_is_boolean(results)) {
+      fprintf(stdout, "%s: Query returned boolean result: %s\n",
+              program,
+              rasqal_query_results_get_boolean(results) ? "true" : "false");
+    }
+    else if (rasqal_query_results_is_graph(results)) {
+      int triple_count=0;
+      
+      if(!quiet)
+        fprintf(stdout, "%s: Query returned graph result:\n", program);
+
+      serializer=raptor_new_serializer(serializer_syntax_name);
+      if(!serializer) {
+        fprintf(stderr, 
+                "%s: Failed to create raptor serializer type %s\n", program,
+                serializer_syntax_name);
+        return(1);
+      }
+
+      raptor_serialize_start_to_file_handle(serializer, base_uri, stdout);
+
+      while(1) {
+        raptor_statement *rs=rasqal_query_results_get_triple(results);
+        if(!rs)
+          break;
+        raptor_serialize_statement(serializer, rs);
+        triple_count++;
+
+        if(rasqal_query_results_next_triple(results))
+          break;
+      }
+
+      raptor_serialize_end(serializer);
+      raptor_free_serializer(serializer);
+
+      if(!quiet)
+        fprintf(stdout, "%s: Total %d triples\n", program, triple_count);
+    } else {
+      fprintf(stdout, "%s: Query returned unknown result format\n", program);
+      rc=1;
     }
   }
 
-  if(!quiet)
-    fprintf(stderr, "%s: Query returned %d results\n", program, 
-            rasqal_query_results_get_count(results));
-  
-
   rasqal_free_query_results(results);
   
  tidy_query:  
@@ -657,14 +729,12 @@ main(int argc, char *argv[])
 
  tidy_setup:
 
-  if(source_uri)
-    raptor_free_uri(source_uri);
+  if(source_uris)
+    raptor_free_sequence(source_uris);
   if(base_uri)
     raptor_free_uri(base_uri);
   if(uri)
     raptor_free_uri(uri);
-  if(free_source_uri_string)
-    raptor_free_memory(source_uri_string);
   if(free_uri_string)
     raptor_free_memory(uri_string);
 
